import sys
from time import sleep
import nidaqmx
from datetime import datetime
from pylablib.devices import Thorlabs

from pymeasure.experiment import Procedure, IntegerParameter, unique_filename, FloatParameter, Parameter
from pymeasure.experiment import Results
from pymeasure.display.Qt import QtWidgets
from pymeasure.display.windows import ManagedWindow
import logging

from src.classes.controltab import ControlTab

log = logging.getLogger('')
log.addHandler(logging.NullHandler())

class MokeProcedure(Procedure):
    iterations = IntegerParameter('Loop Iterations', default=10)
    delay = FloatParameter('Time Step', units='ms', default=2)
    filename = Parameter('Exp name', default="mySampleDesc")

    DATA_COLUMNS = ['Iteration', 'Voltage']

    def startup(self):
        log.info("Setting up the data aquisition")

    def execute(self):
        log.info("Starting the voltage acquisition")

        try:
            nidaqmx.Task()
        except:
            raise RuntimeError("Failed communicating with the DAC, aborting task")

        for i in range(self.iterations):
            with nidaqmx.Task() as task:
                task.ai_channels.add_ai_voltage_chan("Dev1/ai0")
                data = {
                    'Iteration': i,
                    'Voltage': task.read()
                }
            log.debug("Produced numbers: %s" % data)
            self.emit('results', data)
            self.emit('progress', 100 * i / self.iterations)
            sleep(self.delay / 1000.)
            if self.should_stop():
                log.warning("Catch stop command in procedure")
                break

    def shutdown(self):
        log.info("Finished acquisition")


class MainWindow(ManagedWindow):
    def __init__(self):
        super().__init__(
            procedure_class=MokeProcedure,
            inputs=['iterations', 'delay', 'filename'],
            displays=['iterations', 'delay', 'filename'],
            x_axis='Iteration',
            y_axis='Voltage',
            sequencer=True,
            widget_list= tuple([ControlTab("Manual control")]),
            directory_input=True
        )
        self.setWindowTitle('Mini MOKE')
        self.directory = 'data'
        for device in Thorlabs.list_kinesis_devices():
            log.info(f"Thorlab device connected: {device[1]} with serial number: {device[0]}")

    def queue(self, procedure=None):
        direc = self.directory + '/' + datetime.now().strftime('%Y-%m')
        file = unique_filename(direc, 'miniMoke')
        print(self.inputs)

        if procedure is None:
            procedure = self.make_procedure()

        results = Results(procedure, file)
        experiment = self.new_experiment(results)

        self.manager.queue(experiment)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec())